#!/usr/bin/env python
# -*- coding: ISO-8859-1 -*-

"""This wonderful script creates a worthless wxPython window...
When this script is imported, it prevents the parent script from terminating
  until this window is closed.
Alternatively, a script could be run from the shell to acheive the same end.
Another alternative is to call
  app = wx.App(0)
  app.MainLoop()
but this will only work if you are using a wx program like pylab."""

__author__ = "David N. Mashburn <david.n.mashburn@gmail.com>"

# generated by wxGlade 0.4 on Tue Mar 14 23:41:26 2006

import wx

class MyFrame(wx.Frame):
    def __init__(self, *args, **kwds):
        # begin wxGlade: MyFrame.__init__
        kwds["style"] = wx.DEFAULT_FRAME_STYLE
        wx.Frame.__init__(self, *args, **kwds)

        self.__set_properties()
        self.__do_layout()
        # end wxGlade

    def __set_properties(self):
        # begin wxGlade: MyFrame.__set_properties
        self.SetTitle("frame_1")
        # end wxGlade

    def __do_layout(self):
        # begin wxGlade: MyFrame.__do_layout
        sizer_1 = wx.BoxSizer(wx.VERTICAL)
        self.SetAutoLayout(True)
        self.SetSizer(sizer_1)
        sizer_1.Fit(self)
        sizer_1.SetSizeHints(self)
        self.Layout()
        # end wxGlade

# end of class MyFrame

class MyApp(wx.App):
    def OnInit(self):
        wx.InitAllImageHandlers()
        frame_1 = MyFrame(None, -1, "")
        self.SetTopWindow(frame_1)
        frame_1.Show()
        return 1

# end of class MyApp

app = MyApp(0)
app.MainLoop()
